#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : smooth
# @Time         : 2023/12/21 09:40
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 加广告 去广告 平滑

from meutils.pipe import *
from meutils.queues.uniform_queue import UniformQueue

from openai import OpenAI
from openai.types.chat.chat_completion import ChatCompletion
from openai.types.chat.chat_completion_chunk import ChatCompletionChunk
from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk


class Completions(object):

    def __init__(self, **client_params):
        api_key = client_params.get('api_key')
        base_url = client_params.get('base_url')
        self.completions = OpenAI(api_key=api_key, base_url=base_url).chat.completions
        self.interval = client_params.get('interval', 0.015)

        self.slogan = client_params.get('slogan', '')  # 加广告
        self.sub_pattern = client_params.get('sub_pattern')  # 去广告

    def create(self, **data):
        data.pop("additional_kwargs", None)

        response = self.completions.create(**data)
        if data.get('stream'):
            def generator():
                for chunk in response:
                    raw_content = chunk.choices[0].delta.content or ""

                    if self.sub_pattern:
                        raw_content = re.sub(self.sub_pattern, "", raw_content)

                    for content in raw_content:
                        _chunk = chunk.model_copy(deep=True)
                        _chunk.choices[0].delta.content = content
                        yield _chunk

                if len(data.get('messages')) < 16:
                    yield self.chat_completion_chunk_slogan
                else:
                    yield chat_completion_chunk

            return UniformQueue(generator()).consumer(interval=self.interval, break_fn=self.break_fn)

        else:
            response.choices[0].message.content = response.choices[0].message.content + self.slogan
            return response

    def create_sse(self, **data):
        response = self.create(**data)
        if data.get('stream'):
            from sse_starlette import EventSourceResponse
            generator = (chunk.model_dump_json() for chunk in response)
            return EventSourceResponse(generator, ping=10000)
        return response

    @staticmethod
    def break_fn(line: ChatCompletionChunk):
        return line.choices[0].finish_reason

    # @property
    # def chat_completion_slogan(self):
    #     from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk
    #
    #     chat_completion = chat_completion.model_copy(deep=True)
    #     chat_completion.choices[0].message.content = self.slogan
    #     return chat_completion

    @cached_property
    def chat_completion_chunk_slogan(self):
        from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk
        chat_completion_chunk = chat_completion_chunk.model_copy(deep=True)
        chat_completion_chunk.choices[0].delta.content = self.slogan
        return chat_completion_chunk


if __name__ == '__main__':

    data = {'model': 'gpt-3.5-turbo', 'messages': [{'role': 'user', 'content': '你是谁'}], 'stream': True}

    completions = Completions(
        api_key='sk-YpGfgnS3jaevp11Q930aF7F7Ad2a4b689bF23e6fE891B340',
        base_url='http://154.205.7.99:3000/v1',
        slogan='\n\n#### [Xchat永远相信美好的事情即将发生](https://api.chatllm.vip/) ',
    )
    # print(completions.create(**data))
    for i in completions.create(**data):
        print(i.choices[0].delta.content, end='')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ernie
# @Time         : 2023/7/31 13:04
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from pydantic import model_validator
from langchain.chat_models.openai import ChatOpenAI
from langchain.utils import get_from_dict_or_env

from langchain.adapters.openai import convert_message_to_dict
from langchain.schema.messages import BaseMessage

from meutils.pipe import *
from chatllm.llmchain.completions import ErnieBotCompletion


class ErnieBot(ChatOpenAI):
    """
    ernie_api_key = {API Key}:{Secret Key}

    "ernie-bot"
    "ernie-bot-turbo-0725"
    """
    client: Any  #: :meta private:
    model_name: str = Field(default="ernie-bot", alias="model")  # eb-instant，
    openai_api_key: Optional[str] = Field(default=None, alias="ernie_api_key")  # ernie_api_key: Optional[str] = None

    class Config:
        """Configuration for this pydantic object."""

        allow_population_by_field_name = True

    @model_validator(mode='after')
    def validate_environment(cls, values: Dict) -> Dict:
        """Validate that api key and python package exists in environment."""
        # 覆盖 openai_api_key
        values["openai_api_key"] = get_from_dict_or_env(
            values, "ernie_api_key", "ERNIE_API_KEY"
        )

        values["client"] = ErnieBotCompletion

        if values["n"] < 1:
            raise ValueError("n must be at least 1.")
        if values["n"] > 1 and values["streaming"]:
            raise ValueError("n must be 1 when streaming.")
        return values

    @property
    def _llm_type(self) -> str:
        return Path(__file__).name  # 'ernie'

    def get_num_tokens_from_messages(self, messages: List[BaseMessage]) -> int:
        """Calculate num tokens with tiktoken package.

        Official documentation: https://github.com/openai/openai-cookbook/blob/
        main/examples/How_to_format_inputs_to_ChatGPT_models.ipynb"""
        if sys.version_info[1] <= 7:
            return super().get_num_tokens_from_messages(messages)
        model, encoding = self._get_encoding_model()
        tokens_per_message = 3
        tokens_per_name = 1
        num_tokens = 0
        messages_dict = [convert_message_to_dict(m) for m in messages]
        for message in messages_dict:
            num_tokens += tokens_per_message
            for key, value in message.items():
                # Cast str(value) in case the message value is not a string
                # This occurs with function messages
                num_tokens += len(encoding.encode(str(value)))
                if key == "name":
                    num_tokens += tokens_per_name
        # every reply is primed with <im_start>assistant
        num_tokens += 3
        return num_tokens


if __name__ == '__main__':
    from meutils.pipe import *
    from chatllm.llmchain.llms import ErnieBot

    from langchain.chains import LLMChain
    from langchain.prompts import ChatPromptTemplate

    prompt = ChatPromptTemplate.from_template("{q}")

    llm = ErnieBot()
    # c = LLMChain(llm=llm, prompt=prompt)
    # print(c.run('你是谁'))

    prompt_template = """Use the following pieces of context to answer the question at the end. If you don't know the answer, just say that you don't know, don't try to make up an answer.

    {context}

    Question: {question}
    Helpful Answer:"""
    prompt = ChatPromptTemplate.from_template(prompt_template)

    c = LLMChain(llm=llm, prompt=prompt)
    # print(c.run(question='灭霸是谁', context='灭霸是周杰伦'))

    # 计算token
    m = prompt.format_messages(question='灭霸是谁', context='灭霸是周杰伦')
    # m = prompt.format_prompt(question='灭霸是谁', context='灭霸是周杰伦').to_messages()

    print(llm.get_num_tokens_from_messages(m))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatmind
# @Time         : 2023/6/29 15:24
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from chatllm.applications.chatmind import ChatMind

import streamlit as st
from streamlit.components.v1 import html

st.set_page_config(page_title='🔥ChatMind', layout='wide', initial_sidebar_state='collapsed')

api_key = st.sidebar.text_input('API_KEY', 'sk-...')

os.environ['API_KEY'] = api_key
qa = ChatMind()
qa.load_llm()

with st.form('form'):
    title = st.text_input("输入主题", value='人工智能的未来')
    col1, col2 = st.columns(2)
    context: str = ''
    with col1:
        if st.form_submit_button("🚀开始创作"):
            output = st.empty()
            for i in qa(title=title):
                context += i
                output.markdown(context)
    with col2:
        if context:
            html_str = qa.mind_html(context)

            html(html_str, height=1000)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : st_chat
# @Time         : 2023/8/11 14:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

import streamlit as st
# st.set_page_config('🔥ChatLLM', layout='wide', initial_sidebar_state='collapsed')

from langchain import LLMChain, PromptTemplate
from langchain.prompts import ChatPromptTemplate

from meutils.pipe import *

from chatllm.llmchain import init_cache
from chatllm.llmchain.applications import ChatFile
from chatllm.llmchain.document_loaders import FileLoader
from chatllm.llmchain.embeddings import OpenAIEmbeddings
from chatllm.llmchain.decorators import llm_stream

from langchain.chat_models import ChatOpenAI
from streamlit_option_menu import option_menu
from meutils.serving.streamlit.common import hide_st_style

hide_st_style()


init_cache(1)

context_prompt_template = """
根据以下信息，简洁、专业地回答用户的问题。如果无法得到答案，请回复：“对不起，根据已知信息无法回答该问题”或“没有提供足够的信息”。请勿编造信息，答案必须使用中文。

已知信息：
{context}

问题：
{question}

让我们一步一步思考并回答：
""".strip()  # Let's think step by step


class ChatMessage(BaseModel):
    role: str
    content: str


def chat(
    user_role='user',

    assistant_role='assistant',
    assistant_avator="规丞相.png",

    reply_func=lambda input: f'{input}的答案',
    max_turns=3,
    system_prompt="欢迎来找**规丞相**，您有什么要咨询的吗❓"
):
    def chat_message(role):
        if role == 'user':
            return st.chat_message(user_role)
        else:
            return st.chat_message(assistant_role, avatar=assistant_avator)

    # Initialize chat history
    if "messages" not in st.session_state:
        st.session_state.messages = []

    else:
        st.session_state.messages = st.session_state.messages[-2 * (max_turns - 1):]

    # Display chat messages from history on app rerun
    for message in st.session_state.messages:
        with chat_message(message.role):
            st.markdown(message.content, unsafe_allow_html=True)

    container1 = st.container()  # 占位符

    prompt = st.chat_input("    🤔 你可以问我任何问题")

    if prompt:
        print('\n')
        print(prompt)
        prompt = prompt.strip()
        # Display user message in chat message container
        with chat_message(user_role):
            st.markdown(prompt)
        # Add user message to chat history
        st.session_state.messages.append(ChatMessage(role=user_role, content=prompt))

        with chat_message(assistant_role):
            message_placeholder = st.empty()

            response = ''
            gen = reply_func(prompt) or '根据已知信息无法召回相关内容。'
            for token in gen:
                # Display robot response in chat message container
                response += token
                message_placeholder.markdown(response + "▌")
            message_placeholder.markdown(response, unsafe_allow_html=True)

        # Add robot response to chat history
        st.session_state.messages.append(ChatMessage(role=assistant_role, content=response))
    else:
        with chat_message(assistant_role):
            message_placeholder = st.empty()
            message_placeholder.markdown(system_prompt, unsafe_allow_html=True)


@st.cache_resource(show_spinner=False)
def get_reply_func(file):
    reply_func = lambda input: f'### ⚠️请先上传文档\n'

    if file:
        docs = FileLoader(file, file.name).load_and_split()
        print(file.name, len(docs))

        cb = ChatFile(embeddings=OpenAIEmbeddings(chunk_size=20), prompt_template=context_prompt_template)
        cb.create_index(docs)
        print(docs)

        reply_func = lambda query: cb.llm_qa(query=query, k=5, threshold=0.5)

    return reply_func


def chatgpt_reply_func():
    template = ChatPromptTemplate.from_messages([
        ("system", "You are a helpful AI bot. Your name is 东北证券“规丞相”. 请牢记你的身份。"),
        ("human", "{user_input}"),
    ])
    llm = LLMChain(llm=ChatOpenAI(streaming=True), prompt=template)
    reply_func = lambda query: llm_stream(llm.run)(query)
    return reply_func


if __name__ == '__main__':
    col1, col2, col3, *_ = st.columns(3)
    with col1:
        st.image('规丞相.png', width=64)
    with col2:
        st.markdown('##### ')
        st.markdown('##### ')
        st.markdown('##### 规丞相驾到')

    st.markdown('> ⚠️“规丞相”仅供东北证券内部测试使用，所做回答不得用于东北证券官方回复。')

    selected = option_menu("", ["开放式问答", "基于知识库问答"], menu_icon="cast", orientation="horizontal")

    reply_func = None
    if selected == '开放式问答':
        st.session_state.messages = []

        reply_func = chatgpt_reply_func()

    else:
        if hasattr(st.session_state, 'messages'):
            st.session_state.messages = []

        file = st.file_uploader(" ", type=['pdf', 'doc', 'docx', 'txt', ], help='目前仅支持单文档问答')
        if file:
            print(f"{time.ctime()}: {file}")

            with st.spinner('AI正在处理...'):
                reply_func = get_reply_func(file)
        else:
            file = open("东北证券股份有限公司合规手册（东证合规发〔2022〕25号 20221229）.pdf", 'rb')
            reply_func = get_reply_func(file)


    chat(reply_func=reply_func)



# !/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2023/12/21 17:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from urllib.parse import quote, unquote


_ = quote("sk-PB2D9FR6tBNml59tAc72E49982Aa476a9d792b01152a96D7||1")

print(_)

print(unquote(_))
